@@ -18,19 +18,13 @@
 #include <errno.h>
 #include <syslog.h>
 
-#include <common/compat.h>
-#include <common/config.h>
-#include <common/hathreads.h>
-#include <common/ticks.h>
-#include <common/time.h>
-#include <common/tools.h>
-
-#include <types/global.h>
-
-#include <proto/activity.h>
-#include <proto/fd.h>
-#include <proto/log.h>
-#include <proto/signal.h>
+#include <haproxy/activity.h>
+#include <haproxy/api.h>
+#include <haproxy/fd.h>
+#include <haproxy/global.h>
+#include <haproxy/signal.h>
+#include <haproxy/ticks.h>
+#include <haproxy/time.h>
 
 /*
  * Private data:
@@ -114,7 +108,7 @@ static void _update_fd(int fd)
  * "src/fd.c" for more information.
  */
 
-REGPRM3 static void _do_poll(struct poller *p, int exp, int wake)
+static void _do_poll(struct poller *p, int exp, int wake)
 {
 	int i;
 	int wait_time;
@@ -131,7 +125,7 @@ REGPRM3 static void _do_poll(struct poller *p, int exp, int wake)
 
 		_HA_ATOMIC_AND(&fdtab[fd].update_mask, ~tid_bit);
 		if (fdtab[fd].owner == NULL) {
-			activity[tid].poll_drop++;
+			activity[tid].poll_drop_fd++;
 			continue;
 		}
 
@@ -213,19 +207,25 @@ REGPRM3 static void _do_poll(struct poller *p, int exp, int wake)
 
 	thread_harmless_end();
 
+	if (nevlist > 0)
+		activity[tid].poll_io++;
+
 	for (i = 0; i < nevlist; i++) {
 		unsigned int n = 0;
 		int events, rebind_events;
 		fd = evports_evlist[i].portev_object;
 		events = evports_evlist[i].portev_events;
 
+#ifdef DEBUG_FD
+		_HA_ATOMIC_ADD(&fdtab[fd].event_count, 1);
+#endif
 		if (fdtab[fd].owner == NULL) {
-			activity[tid].poll_dead++;
+			activity[tid].poll_dead_fd++;
 			continue;
 		}
 
 		if (!(fdtab[fd].thread_mask & tid_bit)) {
-			activity[tid].poll_skip++;
+			activity[tid].poll_skip_fd++;
 			continue;
 		}
 
@@ -249,7 +249,7 @@ REGPRM3 static void _do_poll(struct poller *p, int exp, int wake)
 		/*
 		 * Call connection processing callbacks.  Note that it's
 		 * possible for this processing to alter the required event
-		 * port assocation; i.e., the "state" member of the "fdtab"
+		 * port association; i.e., the "state" member of the "fdtab"
 		 * entry.  If it changes, the fd will be placed on the updated
 		 * list for processing the next time we are called.
 		 */
@@ -330,7 +330,7 @@ static void deinit_evports_per_thread()
  * Initialisation of the event ports poller.
  * Returns 0 in case of failure, non-zero in case of success.
  */
-REGPRM1 static int _do_init(struct poller *p)
+static int _do_init(struct poller *p)
 {
 	p->private = NULL;
 
@@ -352,7 +352,7 @@ fail:
  * Termination of the event ports poller.
  * All resources are released and the poller is marked as inoperative.
  */
-REGPRM1 static void _do_term(struct poller *p)
+static void _do_term(struct poller *p)
 {
 	if (evports_fd[tid] != -1) {
 		close(evports_fd[tid]);
@@ -372,7 +372,7 @@ REGPRM1 static void _do_term(struct poller *p)
  * the poller to function correctly.
  * Returns 1 on success, otherwise 0.
  */
-REGPRM1 static int _do_test(struct poller *p)
+static int _do_test(struct poller *p)
 {
 	int fd;
 
@@ -389,7 +389,7 @@ REGPRM1 static int _do_test(struct poller *p)
  * otherwise 0.  If this function fails, "_do_term()" must be called to
  * clean up the poller.
  */
-REGPRM1 static int _do_fork(struct poller *p)
+static int _do_fork(struct poller *p)
 {
 	if (evports_fd[tid] != -1) {
 		close(evports_fd[tid]);
@@ -422,7 +422,7 @@ static void _do_register(void)
 
 	p->name = "evports";
 	p->pref = 300;
-	p->flags = 0;
+	p->flags = HAP_POLL_F_ERRHUP;
 	p->private = NULL;
 
 	p->clo  = NULL;
