@@ -10,28 +10,24 @@
  *
  */
 
-#include <common/buffer.h>
-#include <common/debug.h>
-#include <common/cfgparse.h>
-#include <common/compat.h>
-#include <common/config.h>
-#include <common/errors.h>
-#include <common/htx.h>
-#include <common/initcall.h>
-#include <common/namespace.h>
-#include <common/standard.h>
-#include <common/hathreads.h>
+#include <haproxy/api.h>
+#include <haproxy/buf-t.h>
+#include <haproxy/cfgparse.h>
+#include <haproxy/compression.h>
+#include <haproxy/errors.h>
+#include <haproxy/filters.h>
+#include <haproxy/flt_http_comp.h>
+#include <haproxy/http_ana.h>
+#include <haproxy/http_htx.h>
+#include <haproxy/htx.h>
+#include <haproxy/namespace.h>
+#include <haproxy/stream.h>
+#include <haproxy/stream_interface.h>
+#include <haproxy/tools.h>
+#include <haproxy/trace.h>
 
-#include <types/filters.h>
-#include <types/proto_http.h>
 
-#include <proto/compression.h>
-#include <proto/filters.h>
-#include <proto/flt_http_comp.h>
-#include <proto/http_htx.h>
-#include <proto/proto_http.h>
-#include <proto/stream.h>
-#include <proto/stream_interface.h>
+#define TRACE_SOURCE &trace_strm
 
 /* Pool used to allocate filters */
 DECLARE_STATIC_POOL(pool_head_filter, "filter", sizeof(struct filter));
@@ -140,6 +136,9 @@ flt_dump_kws(char **out)
 	struct flt_kw_list *kwl;
 	int index;
 
+	if (!out)
+		return;
+
 	*out = NULL;
 	list_for_each_entry(kwl, &flt_keywords.list, list) {
 		for (index = 0; kwl->kw[index].kw != NULL; index++) {
@@ -292,9 +291,6 @@ flt_init_all()
 	int err_code = 0;
 
 	for (px = proxies_list; px; px = px->next) {
-		if (px->state == PR_STSTOPPED)
-			continue;
-
 		err_code |= flt_init(px);
 		if (err_code & (ERR_ABORT|ERR_FATAL)) {
 			ha_alert("Failed to initialize filters for proxy '%s'.\n",
@@ -314,9 +310,6 @@ flt_init_all_per_thread()
 	int err_code = 0;
 
 	for (px = proxies_list; px; px = px->next) {
-		if (px->state == PR_STSTOPPED)
-			continue;
-
 		err_code = flt_init_per_thread(px);
 		if (err_code & (ERR_ABORT|ERR_FATAL)) {
 			ha_alert("Failed to initialize filters for proxy '%s' for thread %u.\n",
@@ -473,10 +466,6 @@ flt_stream_start(struct stream *s)
 		if (FLT_OPS(filter)->stream_start && FLT_OPS(filter)->stream_start(s, filter) < 0)
 			return -1;
 	}
-	if (strm_li(s) && (strm_li(s)->analysers & AN_REQ_FLT_START_FE)) {
-		s->req.flags |= CF_FLT_ANALYZE;
-		s->req.analysers |= AN_REQ_FLT_END;
-	}
 	return 0;
 }
 
@@ -535,123 +524,10 @@ flt_set_stream_backend(struct stream *s, struct proxy *be)
 		    FLT_OPS(filter)->stream_set_backend(s, filter, be) < 0)
 			return -1;
 	}
-	if (be->be_req_ana & AN_REQ_FLT_START_BE) {
-		s->req.flags |= CF_FLT_ANALYZE;
-		s->req.analysers |= AN_REQ_FLT_END;
-	}
-	if ((strm_fe(s)->fe_rsp_ana | be->be_rsp_ana) & (AN_RES_FLT_START_FE|AN_RES_FLT_START_BE)) {
-		s->res.flags |= CF_FLT_ANALYZE;
-		s->res.analysers |= AN_RES_FLT_END;
-	}
 
 	return 0;
 }
 
-/*
- * Calls 'http_data' callback for all "data" filters attached to a stream. This
- * function is called when incoming data are available (excluding chunks
- * envelope for chunked messages) in the AN_REQ_HTTP_XFER_BODY and
- * AN_RES_HTTP_XFER_BODY analyzers. It takes care to update the next offset of
- * filters and adjusts available data to be sure that a filter cannot parse more
- * data than its predecessors. A filter can choose to not consume all available
- * data. Returns -1 if an error occurs, the number of consumed bytes otherwise.
- *
- * DEPRECATED FUNCTION - CALLED FROM LEGACY HTTP ANALYZERS
- */
-int
-flt_http_data(struct stream *s, struct http_msg *msg)
-{
-	struct filter *filter;
-	unsigned int   buf_i;
-	int            delta = 0, ret = 0;
-
-	/* Save buffer state */
-	buf_i = ci_data(msg->chn);
-
-	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		unsigned int *nxt;
-
-		/* Call "data" filters only */
-		if (!IS_DATA_FILTER(filter, msg->chn))
-			continue;
-
-		/* If the HTTP parser is ahead, we update the next offset of the
-		 * current filter. This happens for chunked messages, at the
-		 * beginning of a new chunk. */
-		nxt = &FLT_NXT(filter, msg->chn);
-		if (msg->next > *nxt)
-			*nxt = msg->next;
-
-		if (FLT_OPS(filter)->http_data) {
-			unsigned int i = ci_data(msg->chn);
-
-			ret = FLT_OPS(filter)->http_data(s, filter, msg);
-			if (ret < 0)
-				break;
-			delta += (int)(ci_data(msg->chn) - i);
-
-			/* Update the next offset of the current filter */
-			*nxt += ret;
-
-			/* And set this value as the bound for the next
-			 * filter. It will not able to parse more data than this
-			 * one. */
-			b_set_data(&msg->chn->buf, co_data(msg->chn) + *nxt);
-		}
-		else {
-			/* Consume all available data and update the next offset
-			 * of the current filter. buf->i is untouched here. */
-			ret = MIN(msg->chunk_len + msg->next, ci_data(msg->chn)) - *nxt;
-			*nxt += ret;
-		}
-	}
-
-	/* Restore the original buffer state */
-	b_set_data(&msg->chn->buf, co_data(msg->chn) + buf_i + delta);
-
-	return ret;
-}
-
-/*
- * Calls 'http_chunk_trailers' callback for all "data" filters attached to a
- * stream. This function is called for chunked messages only when a part of the
- * trailers was parsed in the AN_REQ_HTTP_XFER_BODY and AN_RES_HTTP_XFER_BODY
- * analyzers. Filters can know how much data were parsed by the HTTP parsing
- * until the last call with the msg->sol value. Returns a negative value if an
- * error occurs, any other value otherwise.
- *
- * DEPRECATED FUNCTION - CALLED FROM LEGACY HTTP ANALYZERS
- */
-int
-flt_http_chunk_trailers(struct stream *s, struct http_msg *msg)
-{
-	struct filter *filter;
-	int            ret = 1;
-
-	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		unsigned int *nxt;
-
-		/* Call "data" filters only */
-		if (!IS_DATA_FILTER(filter, msg->chn))
-			continue;
-
-		/* Be sure to set the next offset of the filter at the right
-		 * place. This is really useful when the first part of the
-		 * trailers was parsed. */
-		nxt = &FLT_NXT(filter, msg->chn);
-		*nxt = msg->next;
-
-		if (FLT_OPS(filter)->http_chunk_trailers) {
-			ret = FLT_OPS(filter)->http_chunk_trailers(s, filter, msg);
-			if (ret < 0)
-				break;
-		}
-		/* Update the next offset of the current filter. Here all data
-		 * are always consumed. */
-		*nxt += msg->sol;
-	}
-	return ret;
-}
 
 /*
  * Calls 'http_end' callback for all filters attached to a stream. All filters
@@ -659,142 +535,62 @@ flt_http_chunk_trailers(struct stream *s, struct http_msg *msg)
  * functions is called when all data were parsed and forwarded. 'http_end'
  * callback is resumable, so this function returns a negative value if an error
  * occurs, 0 if it needs to wait for some reason, any other value otherwise.
- *
- * Be carefull, this function can be called from the HTTP legacy analyzers or
- * from HTX analyzers. If your filter is compatible with the two modes, use
- * IS_HTX_STRM macro on the stream.
  */
 int
 flt_http_end(struct stream *s, struct http_msg *msg)
 {
-	unsigned long long *strm_off = &FLT_STRM_OFF(s, msg->chn);
-	unsigned int offset = 0;
 	int ret = 1;
 
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s, s->txn, msg);
 	RESUME_FILTER_LOOP(s, msg->chn) {
-		unsigned long long flt_off = FLT_OFF(filter, msg->chn);
-		offset = flt_off - *strm_off;
-
-		/* Call http_end for data filters only. But the filter offset is
-		 * still valid for all filters
-		 . */
-		if (!IS_DATA_FILTER(filter, msg->chn))
-			continue;
-
 		if (FLT_OPS(filter)->http_end) {
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 			ret = FLT_OPS(filter)->http_end(s, filter, msg);
 			if (ret <= 0)
 				BREAK_EXECUTION(s, msg->chn, end);
 		}
 	} RESUME_FILTER_END;
-
-	c_adv(msg->chn, offset);
-	*strm_off += offset;
-
 end:
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 	return ret;
 }
 
 /*
  * Calls 'http_reset' callback for all filters attached to a stream. This
  * happens when a 100-continue response is received.
- *
- * Be carefull, this function can be called from the HTTP legacy analyzers or
- * from HTX analyzers. If your filter is compatible with the two modes, use
- * IS_HTX_STRM macro on the stream.
  */
 void
 flt_http_reset(struct stream *s, struct http_msg *msg)
 {
 	struct filter *filter;
 
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s, s->txn, msg);
 	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		if (FLT_OPS(filter)->http_reset)
+		if (FLT_OPS(filter)->http_reset) {
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 			FLT_OPS(filter)->http_reset(s, filter, msg);
+		}
 	}
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 }
 
 /*
  * Calls 'http_reply' callback for all filters attached to a stream when HA
  * decides to stop the HTTP message processing.
- *
- * Be carefull, this function can be called from the HTTP legacy analyzers or
- * from HTX analyzers. If your filter is compatible with the two modes, use
- * IS_HTX_STRM macro on the stream.
  */
 void
 flt_http_reply(struct stream *s, short status, const struct buffer *msg)
 {
 	struct filter *filter;
 
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s, s->txn, msg);
 	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		if (FLT_OPS(filter)->http_reply)
+		if (FLT_OPS(filter)->http_reply) {
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 			FLT_OPS(filter)->http_reply(s, filter, status, msg);
-	}
-}
-
-/*
- * Calls 'http_forward_data' callback for all "data" filters attached to a HTTP
- * legacy stream. This function is called when some data can be forwarded in the
- * AN_REQ_HTTP_XFER_BODY and AN_RES_HTTP_XFER_BODY analyzers. It takes care to
- * update the forward offset of filters and adjusts "forwardable" data to be
- * sure that a filter cannot forward more data than its predecessors. A filter
- * can choose to not forward all parsed data. Returns a negative value if an
- * error occurs, else the number of forwarded bytes.
- *
- * DEPRECATED FUNCTION - CALLED FROM LEGACY HTTP ANALYZERS
- */
-int
-flt_http_forward_data(struct stream *s, struct http_msg *msg, unsigned int len)
-{
-	struct filter *filter;
-	int            ret = len;
-
-	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		unsigned int *nxt, *fwd;
-
-		/* Call "data" filters only */
-		if (!IS_DATA_FILTER(filter, msg->chn))
-			continue;
-
-		/* If the HTTP parser is ahead, we update the next offset of the
-		 * current filter. This happens for chunked messages, when the
-		 * chunk envelope is parsed. */
-		nxt = &FLT_NXT(filter, msg->chn);
-		fwd = &FLT_FWD(filter, msg->chn);
-		if (msg->next > *nxt)
-			*nxt = msg->next;
-
-		if (FLT_OPS(filter)->http_forward_data) {
-			/* Remove bytes that the current filter considered as
-			 * forwarded */
-			ret = FLT_OPS(filter)->http_forward_data(s, filter, msg, ret - *fwd);
-			if (ret < 0)
-				goto end;
 		}
-
-		/* Adjust bytes that the current filter considers as
-		 * forwarded */
-		*fwd += ret;
-
-		/* And set this value as the bound for the next filter. It will
-		 * not able to forward more data than the current one. */
-		ret = *fwd;
 	}
-
-	if (!ret)
-		goto end;
-
-	/* Finally, adjust filters offsets by removing data that HAProxy will
-	 * forward. */
-	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		if (!IS_DATA_FILTER(filter, msg->chn))
-			continue;
-		FLT_NXT(filter, msg->chn) -= ret;
-		FLT_FWD(filter, msg->chn) -= ret;
-	}
- end:
-	return ret;
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 }
 
 /*
@@ -805,10 +601,6 @@ flt_http_forward_data(struct stream *s, struct http_msg *msg, unsigned int len)
  * forward more data than its predecessors. A filter can choose to not forward
  * all data. Returns a negative value if an error occurs, else the number of
  * forwarded bytes.
- *
- * Be carefull, this callback is only called from HTX analyzers. So the
- * channel's buffer must be considered as an HTX structured. Of course, your
- * filter must support HTX streams.
  */
 int
 flt_http_payload(struct stream *s, struct http_msg *msg, unsigned int len)
@@ -818,22 +610,17 @@ flt_http_payload(struct stream *s, struct http_msg *msg, unsigned int len)
 	unsigned int out = co_data(msg->chn);
 	int ret, data;
 
-	strm_flt(s)->flags &= ~STRM_FLT_FL_HOLD_HTTP_HDRS;
-
 	ret = data = len - out;
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s, s->txn, msg);
 	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		unsigned long long *flt_off = &FLT_OFF(filter, msg->chn);
-		unsigned int offset = *flt_off - *strm_off;
-
-		/* Call http_payload for filters only. Forward all data for
-		 * others and update the filter offset
-		 */
-		if (!IS_DATA_FILTER(filter, msg->chn)) {
-			*flt_off += data - offset;
+		/* Call "data" filters only */
+		if (!IS_DATA_FILTER(filter, msg->chn))
 			continue;
-		}
-
 		if (FLT_OPS(filter)->http_payload) {
+			unsigned long long *flt_off = &FLT_OFF(filter, msg->chn);
+			unsigned int offset = *flt_off - *strm_off;
+
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 			ret = FLT_OPS(filter)->http_payload(s, filter, msg, out + offset, data - offset);
 			if (ret < 0)
 				goto end;
@@ -842,23 +629,13 @@ flt_http_payload(struct stream *s, struct http_msg *msg, unsigned int len)
 		}
 	}
 
-	/* If nothing was forwarded yet, we take care to hold the headers if
-	 * following conditions are met :
-	 *
-	 *  - *strm_off == 0 (nothing forwarded yet)
-	 *  - ret == 0       (no data forwarded at all on this turn)
-	 *  - STRM_FLT_FL_HOLD_HTTP_HDRS flag set (at least one filter want to hold the headers)
-	 *
-	 * Be careful, STRM_FLT_FL_HOLD_HTTP_HDRS is removed before each http_payload loop.
-	 * Thus, it must explicitly be set when necessary. We must do that to hold the headers
-	 * when there is no payload.
-	 */
-	if (!ret && !*strm_off && (strm_flt(s)->flags & STRM_FLT_FL_HOLD_HTTP_HDRS))
-		goto end;
-
-	ret = data;
-	*strm_off += ret;
+	/* Only forward data if the last filter decides to forward something */
+	if (ret > 0) {
+		ret = data;
+		*strm_off += ret;
+	}
  end:
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 	return ret;
 }
 
@@ -876,12 +653,13 @@ flt_start_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 {
 	int ret = 1;
 
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
+
 	/* If this function is called, this means there is at least one filter,
 	 * so we do not need to check the filter list's emptiness. */
 
 	/* Set flag on channel to tell that the channel is filtered */
 	chn->flags |= CF_FLT_ANALYZE;
-	chn->analysers |= ((chn->flags & CF_ISRESP) ? AN_RES_FLT_END : AN_REQ_FLT_END);
 
 	RESUME_FILTER_LOOP(s, chn) {
 		if (!(chn->flags & CF_ISRESP)) {
@@ -895,10 +673,9 @@ flt_start_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 				continue;
 		}
 
-		FLT_NXT(filter, chn) = 0;
-		FLT_FWD(filter, chn) = 0;
-
+		FLT_OFF(filter, chn) = 0;
 		if (FLT_OPS(filter)->channel_start_analyze) {
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_FLT_ANA, s);
 			ret = FLT_OPS(filter)->channel_start_analyze(s, filter, chn);
 			if (ret <= 0)
 				BREAK_EXECUTION(s, chn, end);
@@ -906,7 +683,9 @@ flt_start_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 	} RESUME_FILTER_END;
 
  end:
-	return handle_analyzer_result(s, chn, an_bit, ret);
+	ret = handle_analyzer_result(s, chn, an_bit, ret);
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
+	return ret;
 }
 
 /*
@@ -924,17 +703,21 @@ flt_pre_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 {
 	int ret = 1;
 
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
+
 	RESUME_FILTER_LOOP(s, chn) {
 		if (FLT_OPS(filter)->channel_pre_analyze && (filter->pre_analyzers & an_bit)) {
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_FLT_ANA, s);
 			ret = FLT_OPS(filter)->channel_pre_analyze(s, filter, chn, an_bit);
 			if (ret <= 0)
 				BREAK_EXECUTION(s, chn, check_result);
-			filter->pre_analyzers &= ~an_bit;
 		}
 	} RESUME_FILTER_END;
 
  check_result:
-	return handle_analyzer_result(s, chn, 0, ret);
+	ret = handle_analyzer_result(s, chn, 0, ret);
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
+	return ret;
 }
 
 /*
@@ -952,66 +735,58 @@ flt_post_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 	struct filter *filter;
 	int            ret = 1;
 
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
+
 	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
 		if (FLT_OPS(filter)->channel_post_analyze &&  (filter->post_analyzers & an_bit)) {
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_FLT_ANA, s);
 			ret = FLT_OPS(filter)->channel_post_analyze(s, filter, chn, an_bit);
 			if (ret < 0)
 				break;
-			filter->post_analyzers &= ~an_bit;
 		}
 	}
-	return handle_analyzer_result(s, chn, 0, ret);
+	ret = handle_analyzer_result(s, chn, 0, ret);
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
+	return ret;
 }
 
 /*
  * This function is the AN_REQ/RES_FLT_HTTP_HDRS analyzer, used to filter HTTP
  * headers or a request or a response. Returns 0 if an error occurs or if it
  * needs to wait, any other value otherwise.
- *
- * Be carefull, this function can be called from the HTTP legacy analyzers or
- * from HTX analyzers. If your filter is compatible with the two modes, use
- * IS_HTX_STRM macro on the stream.
  */
 int
 flt_analyze_http_headers(struct stream *s, struct channel *chn, unsigned int an_bit)
 {
-	struct filter   *filter;
 	struct http_msg *msg;
 	int              ret = 1;
 
 	msg = ((chn->flags & CF_ISRESP) ? &s->txn->rsp : &s->txn->req);
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s, s->txn, msg);
+
 	RESUME_FILTER_LOOP(s, chn) {
 		if (FLT_OPS(filter)->http_headers) {
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
 			ret = FLT_OPS(filter)->http_headers(s, filter, msg);
 			if (ret <= 0)
 				BREAK_EXECUTION(s, chn, check_result);
 		}
 	} RESUME_FILTER_END;
 
-	if (IS_HTX_STRM(s)) {
-		if (HAS_DATA_FILTERS(s, chn)) {
-			size_t data = http_get_hdrs_size(htxbuf(&chn->buf));
-			struct filter *f;
+	if (HAS_DATA_FILTERS(s, chn)) {
+		size_t data = http_get_hdrs_size(htxbuf(&chn->buf));
+		struct filter *f;
 
-			list_for_each_entry(f, &strm_flt(s)->filters, list)
+		list_for_each_entry(f, &strm_flt(s)->filters, list) {
+			if (IS_DATA_FILTER(f, chn))
 				FLT_OFF(f, chn) = data;
 		}
 	}
-	else {
-		/* We increase next offset of all "data" filters after all processing on
-		 * headers because any filter can alter them. So the definitive size of
-		 * headers (msg->sov) is only known when all filters have been
-		 * called. */
-		list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-			/* Handle "data" filters only */
-			if (!IS_DATA_FILTER(filter, chn))
-				continue;
-			FLT_NXT(filter, chn) = msg->sov;
-		}
-	}
 
  check_result:
-	return handle_analyzer_result(s, chn, an_bit, ret);
+	ret = handle_analyzer_result(s, chn, an_bit, ret);
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_HTTP_ANA|STRM_EV_FLT_ANA, s);
+	return ret;
 }
 
 /*
@@ -1026,17 +801,19 @@ flt_end_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 {
 	int ret = 1;
 
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
+
 	/* Check if all filters attached on the stream have finished their
 	 * processing on this channel. */
 	if (!(chn->flags & CF_FLT_ANALYZE))
 		goto sync;
 
 	RESUME_FILTER_LOOP(s, chn) {
-		FLT_NXT(filter, chn) = 0;
-		FLT_FWD(filter, chn) = 0;
+		FLT_OFF(filter, chn) = 0;
 		unregister_data_filter(s, chn, filter);
 
 		if (FLT_OPS(filter)->channel_end_analyze) {
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_FLT_ANA, s);
 			ret = FLT_OPS(filter)->channel_end_analyze(s, filter, chn);
 			if (ret <= 0)
 				BREAK_EXECUTION(s, chn, end);
@@ -1070,129 +847,60 @@ flt_end_analyze(struct stream *s, struct channel *chn, unsigned int an_bit)
 		s->req.analysers &= ~AN_REQ_FLT_END;
 		s->res.analysers &= ~AN_RES_FLT_END;
 
-		/* Clean up the HTTP transaction if needed */
-		if (s->txn && (s->txn->flags & TX_WAIT_CLEANUP))
-			http_end_txn_clean_session(s);
-
 		/* Remove backend filters from the list */
 		flt_stream_release(s, 1);
+		DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
+	}
+	else {
+		DBG_TRACE_DEVEL("waiting for sync", STRM_EV_STRM_ANA|STRM_EV_FLT_ANA, s);
 	}
-
 	return ret;
 }
 
 
 /*
- * Calls 'tcp_data' callback for all "data" filters attached to a stream. This
- * function is called when incoming data are available. It takes care to update
- * the next offset of filters and adjusts available data to be sure that a
- * filter cannot parse more data than its predecessors. A filter can choose to
- * not consume all available data. Returns -1 if an error occurs, the number of
- * consumed bytes otherwise.
+ * Calls 'tcp_payload' callback for all "data" filters attached to a
+ * stream. This function is called when some data can be forwarded in the
+ * AN_REQ_FLT_XFER_BODY and AN_RES_FLT_XFER_BODY analyzers. It takes care to
+ * update the filters and the stream offset to be sure that a filter cannot
+ * forward more data than its predecessors. A filter can choose to not forward
+ * all data. Returns a negative value if an error occurs, else the number of
+ * forwarded bytes.
  */
-static int
-flt_data(struct stream *s, struct channel *chn)
+int
+flt_tcp_payload(struct stream *s, struct channel *chn, unsigned int len)
 {
 	struct filter *filter;
-	unsigned int   buf_i;
-	int            delta = 0, ret = 0;
-
-	/* Save buffer state */
-	buf_i = ci_data(chn);
+	unsigned long long *strm_off = &FLT_STRM_OFF(s, chn);
+	unsigned int out = co_data(chn);
+	int ret, data;
 
+	ret = data = len - out;
+	DBG_TRACE_ENTER(STRM_EV_TCP_ANA|STRM_EV_FLT_ANA, s);
 	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		unsigned int *nxt;
-
 		/* Call "data" filters only */
 		if (!IS_DATA_FILTER(filter, chn))
 			continue;
+		if (FLT_OPS(filter)->tcp_payload) {
+			unsigned long long *flt_off = &FLT_OFF(filter, chn);
+			unsigned int offset = *flt_off - *strm_off;
 
-		nxt = &FLT_NXT(filter, chn);
-		if (FLT_OPS(filter)->tcp_data) {
-			unsigned int i = ci_data(chn);
-
-			ret = FLT_OPS(filter)->tcp_data(s, filter, chn);
-			if (ret < 0)
-				break;
-			delta += (int)(ci_data(chn) - i);
-
-			/* Increase next offset of the current filter */
-			*nxt += ret;
-
-			/* And set this value as the bound for the next
-			 * filter. It will not able to parse more data than the
-			 * current one. */
-			b_set_data(&chn->buf, co_data(chn) + *nxt);
-		}
-		else {
-			/* Consume all available data */
-			*nxt = ci_data(chn);
-		}
-
-		/* Update <ret> value to be sure to have the last one when we
-		 * exit from the loop. This value will be used to know how much
-		 * data are "forwardable" */
-		ret = *nxt;
-	}
-
-	/* Restore the original buffer state */
-	b_set_data(&chn->buf, co_data(chn) + buf_i + delta);
-
-	return ret;
-}
-
-/*
- * Calls 'tcp_forward_data' callback for all "data" filters attached to a
- * stream. This function is called when some data can be forwarded. It takes
- * care to update the forward offset of filters and adjusts "forwardable" data
- * to be sure that a filter cannot forward more data than its predecessors. A
- * filter can choose to not forward all parsed data. Returns a negative value if
- * an error occurs, else the number of forwarded bytes.
- */
-static int
-flt_forward_data(struct stream *s, struct channel *chn, unsigned int len)
-{
-	struct filter *filter;
-	int            ret = len;
-
-	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		unsigned int *fwd;
-
-		/* Call "data" filters only */
-		if (!IS_DATA_FILTER(filter, chn))
-			continue;
-
-		fwd = &FLT_FWD(filter, chn);
-		if (FLT_OPS(filter)->tcp_forward_data) {
-			/* Remove bytes that the current filter considered as
-			 * forwarded */
-			ret = FLT_OPS(filter)->tcp_forward_data(s, filter, chn, ret - *fwd);
+			DBG_TRACE_DEVEL(FLT_ID(filter), STRM_EV_TCP_ANA|STRM_EV_FLT_ANA, s);
+			ret = FLT_OPS(filter)->tcp_payload(s, filter, chn, out + offset, data - offset);
 			if (ret < 0)
 				goto end;
+			data = ret + *flt_off - *strm_off;
+			*flt_off += ret;
 		}
-
-		/* Adjust bytes that the current filter considers as
-		 * forwarded */
-		*fwd += ret;
-
-		/* And set this value as the bound for the next filter. It will
-		 * not able to forward more data than the current one. */
-		ret = *fwd;
 	}
 
-	if (!ret)
-		goto end;
-
-	/* Finally, adjust filters offsets by removing data that HAProxy will
-	 * forward. */
-	list_for_each_entry(filter, &strm_flt(s)->filters, list) {
-		if (!IS_DATA_FILTER(filter, chn))
-			continue;
-		FLT_NXT(filter, chn) -= ret;
-		FLT_FWD(filter, chn) -= ret;
+	/* Only forward data if the last filter decides to forward something */
+	if (ret > 0) {
+		ret = data;
+		*strm_off += ret;
 	}
-
  end:
+	DBG_TRACE_LEAVE(STRM_EV_TCP_ANA|STRM_EV_FLT_ANA, s);
 	return ret;
 }
 
@@ -1207,10 +915,13 @@ flt_forward_data(struct stream *s, struct channel *chn, unsigned int len)
 int
 flt_xfer_data(struct stream *s, struct channel *chn, unsigned int an_bit)
 {
+	unsigned int len;
 	int ret = 1;
 
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_TCP_ANA|STRM_EV_FLT_ANA, s);
+
 	/* If there is no "data" filters, we do nothing */
-	if (!HAS_DATA_FILTERS(s, chn) || (s->flags & SF_HTX))
+	if (!HAS_DATA_FILTERS(s, chn))
 		goto end;
 
 	/* Be sure that the output is still opened. Else we stop the data
@@ -1219,33 +930,40 @@ flt_xfer_data(struct stream *s, struct channel *chn, unsigned int an_bit)
 	    ((chn->flags & CF_SHUTW) && (chn->to_forward || co_data(chn))))
 		goto end;
 
-	/* Let all "data" filters parsing incoming data */
-	ret = flt_data(s, chn);
+	if (s->flags & SF_HTX) {
+		struct htx *htx = htxbuf(&chn->buf);
+		len = htx->data;
+	}
+	else
+		len = c_data(chn);
+
+	ret = flt_tcp_payload(s, chn, len);
 	if (ret < 0)
 		goto end;
-
-	/* And forward them */
-	ret = flt_forward_data(s, chn, ret);
-	if (ret < 0)
-		goto end;
-
-	/* Consume data that all filters consider as forwarded. */
 	c_adv(chn, ret);
 
 	/* Stop waiting data if the input in closed and no data is pending or if
 	 * the output is closed. */
-	if ((chn->flags & CF_SHUTW) ||
-	    ((chn->flags & CF_SHUTR) && !ci_data(chn))) {
+	if (chn->flags & CF_SHUTW) {
 		ret = 1;
 		goto end;
 	}
+	if (chn->flags & CF_SHUTR) {
+		if (((s->flags & SF_HTX) && htx_is_empty(htxbuf(&chn->buf))) || c_empty(chn)) {
+			ret = 1;
+			goto end;
+		}
+	}
 
 	/* Wait for data */
+	DBG_TRACE_DEVEL("waiting for more data", STRM_EV_STRM_ANA|STRM_EV_TCP_ANA|STRM_EV_FLT_ANA, s);
 	return 0;
  end:
 	/* Terminate the data filtering. If <ret> is negative, an error was
 	 * encountered during the filtering. */
-	return handle_analyzer_result(s, chn, an_bit, ret);
+	ret = handle_analyzer_result(s, chn, an_bit, ret);
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_TCP_ANA|STRM_EV_FLT_ANA, s);
+	return ret;
 }
 
 /*
@@ -1289,7 +1007,7 @@ handle_analyzer_result(struct stream *s, struct channel *chn,
 		/* FIXME: incr counters */
 	}
 
-	if (s->txn) {
+	if (IS_HTX_STRM(s)) {
 		/* Do not do that when we are waiting for the next request */
 		if (s->txn->status > 0)
 			http_reply_and_close(s, s->txn->status, NULL);
@@ -1303,11 +1021,13 @@ handle_analyzer_result(struct stream *s, struct channel *chn,
 		s->flags |= SF_ERR_PRXCOND;
 	if (!(s->flags & SF_FINST_MASK))
 		s->flags |= finst;
+	DBG_TRACE_DEVEL("leaving on error", STRM_EV_FLT_ANA|STRM_EV_FLT_ERR, s);
 	return 0;
 
  wait:
 	if (!(chn->flags & CF_ISRESP))
 		channel_dont_connect(chn);
+	DBG_TRACE_DEVEL("wairing for more data", STRM_EV_FLT_ANA, s);
 	return 0;
 }
 
