@@ -396,10 +396,12 @@ int http_replace_req_uri(struct htx *htx, const struct ist uri)
 	return 0;
 }
 
-/* Replace the request path in the HTX message <htx> by <path>. The host part
- * and the query string are preserved. It returns 1 on success, otherwise 0.
+/* Replace the request path in the HTX message <htx> by <path>. The host part is
+ * preserverd. if <with_qs> is set, the query string is evaluated as part of the
+ * path and replaced. Otherwise, it is preserved too. It returns 1 on success,
+ * otherwise 0.
  */
-int http_replace_req_path(struct htx *htx, const struct ist path)
+int http_replace_req_path(struct htx *htx, const struct ist path, int with_qs)
 {
 	struct buffer *temp = get_trash_chunk();
 	struct htx_sl *sl = http_get_stline(htx);
@@ -413,8 +415,12 @@ int http_replace_req_path(struct htx *htx, const struct ist path)
 	p = http_get_path(uri);
 	if (!isttest(p))
 		p = uri;
-	while (plen < p.len && *(p.ptr + plen) != '?')
-		plen++;
+	if (with_qs)
+		plen = p.len;
+	else {
+		while (plen < p.len && *(p.ptr + plen) != '?')
+			plen++;
+	}
 
 	/* Start by copying old method and version and create the new uri */
 	chunk_memcat(temp, HTX_SL_REQ_MPTR(sl), HTX_SL_REQ_MLEN(sl)); /* meth */
@@ -2618,17 +2624,17 @@ int val_blk_arg(struct arg *arg, char **err_msg)
 		return 0;
 	}
 	if (arg[0].data.str.data == 4 && !strncmp(arg[0].data.str.area, "head", 4)) {
-		free(arg[0].data.str.area);
+		chunk_destroy(&arg[0].data.str);
 		arg[0].type = ARGT_SINT;
 		arg[0].data.sint = -1;
 	}
 	else if (arg[0].data.str.data == 4 && !strncmp(arg[0].data.str.area, "tail", 4)) {
-		free(arg[0].data.str.area);
+		chunk_destroy(&arg[0].data.str);
 		arg[0].type = ARGT_SINT;
 		arg[0].data.sint = -2;
 	}
 	else if (arg[0].data.str.data == 5 && !strncmp(arg[0].data.str.area, "first", 5)) {
-		free(arg[0].data.str.area);
+		chunk_destroy(&arg[0].data.str);
 		arg[0].type = ARGT_SINT;
 		arg[0].data.sint = -3;
 	}
@@ -2647,7 +2653,7 @@ int val_blk_arg(struct arg *arg, char **err_msg)
 			memprintf(err_msg, "block position must not be negative");
 			return 0;
 		}
-		free(arg[0].data.str.area);
+		chunk_destroy(&arg[0].data.str);
 		arg[0].type = ARGT_SINT;
 		arg[0].data.sint = pos;
 	}
